<?php

if (!defined('_PS_VERSION_'))
	exit;

class WidgetForms
{
	protected static $module = false;

	public static function init($module)
	{
		if (self::$module == false)
			self::$module = $module;

		return self::$module;
	}


	public static function getWidgetsList()
	{
		return array(
			'id_pswidget' => array('title' => self::$module->l('Id', 'WidgetForms'), 'type' => 'text', 'orderby' => true),
			'page_type' => array('title' => self::$module->l('Page type', 'WidgetForms'), 'type' => 'text', 'orderby' => false),
			'widget_id' => array('title' => self::$module->l('Widget ID', 'WidgetForms'), 'type' => 'text', 'orderby' => false),
			'widget_enable' => array(
				'title' => self::$module->l('Active', 'WidgetForms'),
				'active' => 'status',
				'type' => 'bool',
				'align' => 'center',
				'orderby' => false
			),
		);
	}


	public static function getWidgetsListValues()
	{
		$widgets = Db::getInstance()->executeS('SELECT * FROM `'._DB_PREFIX_.self::$module->name.'`');
		return $widgets;
	}


	public static function getWidgetForm($title = 'New widget')
	{

		$v2_enabled = (Tools::getValue('v2_enabled') === 'true' ? true : false);

		$form = array(
			array(
				'form' => array(
					'legend' => array(
						'title' => self::$module->l($title),
						'icon' => 'icon-plus',
					),
					'input' => array(),
					'submit' => array('title' => self::$module->l('Save', 'WidgetForms'), 'type' => 'submit', 'class' => 'btn btn-default pull-right'),
				),
			),
		);

		$form[0]['form']['input'][] = array(
			'type' => 'switch',
			'label' => self::$module->l('Active'),
			'name' => 'widget_enable',
			'required' => true,
			'is_bool' => true,
			'values' => array(
				array('id' => 'widget_enable_true', 'value' => 1, 'label' => self::$module->l('Yes')),
				array('id' => 'widget_enable_false', 'value' => 0, 'label' => self::$module->l('No')),
			)
		);
		

		$form[0]['form']['input'][] = array(
			'type' => 'text',
			'name' => 'user_name',
			'required' => true,
			'label' => self::$module->l('User Name', 'WidgetForms'),
			'desc' => self::$module->l('Your Photoslurp user name.', 'WidgetForms'),
		);

		$form[0]['form']['input'][] = array(
			'type' => 'text',
			'name' => 'widget_id',
			'required' => true,
			'label' => self::$module->l('Widget Id', 'WidgetForms'),
			'desc' => self::$module->l('A unique identifier for each of the widgets you use on your website.', 'WidgetForms'),
		);

		$form[0]['form']['input'][] = array(
			'type' => 'select',
			'name' => 'page_type',
            'required' => true,
			'label' => self::$module->l('Page Type', 'WidgetForms'),
			'desc' => self::$module->l('This parameter is used in analytics calculations.', 'WidgetForms'),
			'options' => array(
				'query' => self::getPageTypeOptions(),
				'id' => 'id', 'name' => 'name'
			)
		);
        
        $form[0]['form']['input'][] = array(
			'type' => 'select',
			'name' => 'product_hook',
            'required' => true,
			'label' => self::$module->l('Product Hook', 'WidgetForms'),
			'desc' => self::$module->l('Select a hook of product.', 'WidgetForms'),
			'options' => array(
				'query' => self::getProductHooks(),
				'id' => 'id', 'name' => 'name'
			),
		);
        
        $form[0]['form']['input'][] = array(
			'type' => 'select',
			'name' => 'show_in_category',
			'label' => self::$module->l('Display in Categories', 'WidgetForms'),
			'desc' => self::$module->l('Choose whether you want this widget to be displayed in all category pages or in a custom selection of category pages.', 'WidgetForms'),
			'options' => array(
				'query' => self::getShowCategoryOptions(),
				'id' => 'id', 'name' => 'name'
			)
		);
        
        $form[0]['form']['input'][] = array(
			'type' => 'text',
			'name' => 'identify_list_content',
			'label' => self::$module->l('Content identify', 'WidgetForms'),
			'desc' => self::$module->l('Unique identifier of the container where the widget will appear.', 'WidgetForms'),
		);
        
        $form[0]['form']['input'][] = array(
			'type' => 'select',
			'name' => 'show_in_category_ids',
            'multiple' => true,
			'label' => self::$module->l('Selected Categories', 'WidgetForms'),
			'desc' => self::$module->l('To select multiple categories hold down the Ctrl-key on Windows or the Command-key on Mac while clicking on the category names. To select a range: Click the first category, then hold down the Shift-key and then click the last category in the range', 'WidgetForms'),
            'class'    => 'show_categories_ids',
			'options' => array(
				'query' => self::getCategoryOptions(),
				'id' => 'id', 'name' => 'name'
			),
		);
        
        $form[0]['form']['input'][] = array(
			'type' => 'switch',
			'name' => 'media_from_category',
            'required' => true,
			'label' => self::$module->l('Only Show Media From Product Category', 'WidgetForms'),
			'values' => array(
				array('id' => 'media_from_category_true', 'value' => 1, 'label' => self::$module->l('Yes')),
				array('id' => 'media_from_category_false', 'value' => 0, 'label' => self::$module->l('No')),
			)
		);
        
        $form[0]['form']['input'][] = array(
			'type' => 'select',
			'name' => 'media_from_category_ids',
            'multiple' => true,
            'required' => true,
			'label' => self::$module->l('Categories', 'WidgetForms'),
			'desc' => self::$module->l('Select one or more categories. Press Ctrl to select multiple categories', 'WidgetForms'),
            'class'    => 'media_categories_ids',
			'options' => array(
				'query' => self::getCategoryOptions(),
				'id' => 'id', 'name' => 'name'
			),
		);

        $options=array();
        foreach(Shop::getShops() as $shop){
          $options[] = array(
                "id_option" => (int)$shop['id_shop'],
                'val' => '1',
                "name" => $shop['name'],
              'ckecked' => true
            ); 
        }                
        $form[0]['form']['input'][] = array(
            'type'    => 'checkbox',                   
            'label'   => self::$module->l('Assigned store'),         
            'desc'    => self::$module->l('Choose the store/s where you want to show this widget.'),
            'name'    => 'shop',
            'values'  => array(
                'query' => $options,                   
                'id'    => 'id_option',           
                'value' => 'value',        
                'name'  => 'name',
                'ckecked' => true
            ),
        );
        
        $form[0]['form']['input'][] = array(
			'type' => 'textarea',
			'name' => 'style_custom',
			'label' => self::$module->l('Custom Style', 'WidgetForms'),
			'desc' => self::$module->l('Enter your custom CSS here', 'WidgetForms')
		);

		return $form;
	}

	protected static function getPageTypeOptions(){
		$data = array(
			array('id' => 'home', 'name' => self::$module->l('home', 'WidgetForms')),
			array('id' => 'product', 'name' => self::$module->l('product', 'WidgetForms')),
			array('id' => 'category', 'name' => self::$module->l('category', 'WidgetForms')),
			array('id' => 'lookbook', 'name' => self::$module->l('lookbook', 'WidgetForms'))
		);
		return $data;
	}
    
    public static function getShowCategoryOptions(){
        $data = array(
			array('id' => '1', 'name' => self::$module->l('All categories', 'WidgetForms')),
			array('id' => '2', 'name' => self::$module->l('Only in selected categories', 'WidgetForms'))
		);
		return $data;
    }
    
    public static function getProductHooks(){

		$data = array(
			array('id' => 'left', 'name' => self::$module->l('New elements on the product page (left column)', 'WidgetForms')),
			array('id' => 'right', 'name' => self::$module->l('New elements on the product page (right column)', 'WidgetForms')),
            array('id' => 'product_tab_content', 'name' => self::$module->l('Tabs content on the product page', 'WidgetForms')),
            array('id' => 'product_content', 'name' => self::$module->l('Product Content', 'WidgetForms')),
            array('id' => 'footer', 'name' => self::$module->l('Product Footer', 'WidgetForms'))
		);
		return $data;
	}

	public static function getUpdateWidgetFormValues()
	{
		$id_pswidget = (int)Tools::getValue('id_pswidget');
		$values = array();
		$widget = Db::getInstance()->getRow('SELECT * FROM `'._DB_PREFIX_.self::$module->name.'` WHERE `id_pswidget` = \''.$id_pswidget.'\'');

		$fields = Db::getInstance()->executeS('SHOW COLUMNS FROM '._DB_PREFIX_.self::$module->name);
		foreach ($fields as $field){
            if($field['Field'] == "media_from_category_ids"){
                $values[$field['Field']."[]"] = Tools::getValue($field['Field'], explode(",", $widget[$field['Field']]));
            }elseif($field['Field'] == "show_in_category_ids"){
                $values[$field['Field']."[]"] = Tools::getValue($field['Field'], explode(",", $widget[$field['Field']]));
            }else{
                $values[$field['Field']] = Tools::getValue($field['Field'], $widget[$field['Field']]);
            }
        }

        $widget_shops = Db::getInstance()->executeS('SELECT * FROM `'._DB_PREFIX_.self::$module->name.'_shop` WHERE `id_pswidget` = \''.$id_pswidget.'\'');
             
        foreach ($widget_shops as $widget_shop){           
			$values['shop_'.$widget_shop['id_shop']] = 1;
        }

		return $values;
	}

    public static function getCategoryOptions(){
        $context = Context::getContext();
        $sql = '
            SELECT * 
            FROM '._DB_PREFIX_.'category c
            INNER JOIN '._DB_PREFIX_.'category_lang cl ON(cl.id_category = c.id_category AND cl.id_lang = '.$context->language->id.' AND cl.id_shop = '.$context->shop->id.')
            ORDER BY c.level_depth
        ';
        $result = Db::getInstance()->ExecuteS($sql);
        
        $categories = array();
        $buff = array();

        foreach ($result as $row) {
            $current = &$buff[$row['id_category']];
            $current = $row;
            //Cambio de 0 por un 1 para que encuentre la categoria root 'because yes'
            if ($row['id_parent'] == 1) {
                $categories[$row['id_category']] = &$current;
            } else {
                $buff[$row['id_parent']]['children'][$row['id_category']] = &$current;
            }
        }
        $category_options = self::generateCategoriesOption($categories);
        return $category_options;
    }
    public static function generateCategoriesOption($categories) {
        $options = array();

        foreach ($categories as $key => $category) {
            $options[] = array("id" => (int) $category['id_category'], 'name' => str_repeat('&nbsp;', 4 * (int) $category['level_depth']) . $category['name']);
            if (isset($category['children']) && !empty($category['children'])) {
                $options = array_merge($options, self::generateCategoriesOption($category['children']));
            }
        }
        return $options;
    }
    public static function getNewWidgetFormValues(){

		$values = array();
		$fields = Db::getInstance()->executeS('SHOW COLUMNS FROM '._DB_PREFIX_.self::$module->name);
		foreach ($fields as $field){
            if($field['Field'] == "media_from_category_ids"){
                $values[$field['Field']."[]"] = array();
            }elseif($field['Field'] == "show_in_category_ids"){
                $values[$field['Field']."[]"] = array();
            }else{
                $values[$field['Field']] = "";
            }
        }

		return $values;
	}
}
