<?php

if (!defined('_PS_VERSION_'))
    exit;

require_once(dirname(__FILE__).'/classes/WidgetForms.php');
require_once(dirname(__FILE__).'/classes/Export.php');

class Pswidget extends Module
{
    protected $config_form = false;

    protected $_successes;
    protected $_warnings;
    protected $export_class;

    public function __construct()
    {
        $this->name = 'pswidget';
        $this->tab = 'front_office_features';
        $this->version = '3.1.1';
        $this->author = 'Photoslurp - A Visual Commerce and Marketing platform';
        $this->need_instance = 1;

        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Photoslurp');
        $this->description = $this->l('Set up Photoslurp catalogs and widgets in your store.');
        $this->confirmUninstall = $this->l('Are you sure you want to uninstall Photoslurp Widget?');
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        include(dirname(__FILE__).'/sql/install.php');

        return parent::install() &&
        $this->registerHook('header') &&
        $this->registerHook('backOfficeHeader') &&
        $this->registerHook('displayFooter') &&
        $this->registerHook('displayFooterProduct') &&
        $this->registerHook('displayHeader') &&
        $this->registerHook('displayHome') &&
        $this->registerHook('displayOrderConfirmation') &&
        $this->registerHook('psWidget') &&
        $this->registerHook('displayLeftColumnProduct')&&
        $this->registerHook('displayRightColumnProduct')&&
        $this->registerHook('displayProductTabContent')&&
        $this->registerHook('displayProductContent');

    }


    public function uninstall()
    {
        Configuration::deleteByName('PSWIDGET_PRODUCT_ID_TYPE');
        Configuration::deleteByName('PSWIDGET_EXPORTED_SHOPS');
        Configuration::deleteByName('PSWIDGET_ClICKNSHOPIT');
        foreach(Shop::getShops() as $shop){
            Configuration::deleteByName("PSWIDGET_SHOP_" . $shop['id_shop'] . "_CAMPAIGN");
        }
        include(dirname(__FILE__).'/sql/uninstall.php');
        return (parent::uninstall() && $this->unregisterHook('psWidget'));
    }

    public function getContent()
    {
        $output = null;
        WidgetForms::init($this);

        if (((bool)Tools::isSubmit('submitPswidgetModule')) == true){
            $this->postProcess();
        } elseif ((bool)Tools::isSubmit('submitNewWidget') == true){
            $submit_widget = $this->postProcessNewWidget();
        } elseif ((bool)Tools::isSubmit('submitUpdateWidget') == true){
            $submit_widget = $this->postProcessUpdateWidget();
        } elseif ((bool)Tools::isSubmit('submitExport') == true){
            $this->processExport();
        }

        $this->context->smarty->assign(array(
            'module_dir' => $this->_path,
            'module_local_dir' => $this->local_path,
        ));

        $this->context->smarty->assign('form_errors', $this->_errors);
        $this->context->smarty->assign('form_infos', $this->_warnings);
        $this->context->smarty->assign('form_successes', $this->_successes);

        $admin_url = Tools::gethttphost(true).str_replace(_PS_CORE_DIR_, '', _PS_ADMIN_DIR_).'/';

        if ((Tools::isSubmit('submitNewWidget') || Tools::isSubmit('newpswidget') || Tools::isSubmit('updatepswidget')) && ((isset($submit_widget) == false) || ($submit_widget === false))){
            $back_url = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name.'&token='.Tools::getAdminTokenLite('AdminModules');
        }

        if (Tools::isSubmit('newpswidget') || ((isset($submit_widget) == true) && ($submit_widget === false))){
            $output = $output.$this->renderForm(WidgetForms::getWidgetForm(), WidgetForms::getNewWidgetFormValues(), 'submitNewWidget', true, $back_url);
        } elseif (Tools::isSubmit('updatepswidget') && Tools::isSubmit('id_pswidget')){

            $form_structure = WidgetForms::getWidgetForm('Update widget', true);

            $form = $this->renderForm($form_structure, WidgetForms::getUpdateWidgetFormValues(), 'submitUpdateWidget', true, false, true);

            $output = $output.$form;

        } elseif (Tools::isSubmit('deletepswidget') && Tools::isSubmit('id_pswidget')){
            $this->postProcessDeleteWidget((int)Tools::getValue('id_pswidget'));
        } elseif (defined('_PS_HOST_MODE_') == false){
            if(empty($this->export_class)){
                $this->export_class = new Export(Shop::getShops(), $this->local_path.'csv/');
            }
            $this->context->smarty->assign(array(
                'all_shops_active' => Shop::getShops(),
                'all_files' => (!is_null($this->export_class) ? $this->export_class->getChecked_files($this->export_class->getCsvFiles()) : array()),
                // Export queries set alias sku for product_id field
                'product_id_types' => array(
                    'sku' => 'ID',
                    'ean13' => 'EAN13',
                    'upc' => 'UPC',
                    'reference' => 'REFERENCE',
                ),
                'selected_id_type' => Configuration::get('PSWIDGET_PRODUCT_ID_TYPE')
            ));
            $this->context->smarty->assign(array(
                'form_action' => $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name.'&token='.Tools::getAdminTokenLite('AdminModules'),
                'export_values' => explode(",", Configuration::get('PSWIDGET_EXPORTED_SHOPS'))
            ));
            $output = $output.$this->context->smarty->fetch($this->local_path.'views/templates/admin/export.tpl');
        }
        $this->context->controller->addJS($this->_path.'views/js/back.js?v=11');
        $this->context->controller->addCSS($this->_path.'views/css/back.css');
        $output = $output.$this->renderConfigForm();

        return $output.$this->renderWidgetsList();
    }


    public function hookBackOfficeHeader(){
        if (Tools::getValue('module_name') == $this->name){
            $this->context->controller->addJquery();
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader(){
        if(Configuration::get('PSWIDGET_ClICKNSHOPIT')){
            return $this->display(__FILE__, 'views/templates/front/clicknshopit.tpl');
        }
    }

    protected function getJson($widget){

        $parameters = array();

        unset($widget['id_pswidget']);
        unset($widget['widget_enable']);
        unset($widget['user_name']);
        unset($widget['show_in_category']);
        unset($widget['show_in_category_ids']);
        //unset($widget['media_from_category']);
        //unset($widget['media_from_category_ids']);
        unset($widget['product_hook']);

        foreach($widget as $path=>$value){
            if(($value !== null)){
                if(strpos($path,'style_')=== 0){
                    $temp = &$parameters;
                    foreach(explode('_',$path) as $key) {
                        $temp = &$temp[$key];
                    }
                    $temp = $value;
                    unset($temp);
                }else{
                    $jsName = preg_replace_callback(
                        '/_([^_])/',
                        function (array $m) {
                            return ucfirst($m[1]);
                        },
                        $path
                    );
                    $parameters[$jsName] = $value;
                }
            }
        }

        $parameters['lang'] = $this->context->language->iso_code . '_' .$this->context->shop->id;

        if($widget['media_from_category'] == 1){
            if($widget['page_type'] == 'home' || $widget['page_type'] == 'lookbook' ){
                $idcategories = explode(",", $widget['media_from_category_ids']);
                $parameters['productType'] = array();
                foreach($idcategories as $idcategory){
                    $parameters['productType'][] = $this->getCategoryBreadcrumb($idcategory);
                }
            }
            if($widget['page_type'] == 'product'){
                $parameters['productType'] = $this->getProductTypes();
            }
        }
        unset($parameters['mediaFromCategory']);
        unset($parameters['mediaFromCategoryIds']);
        return json_encode($parameters);
    }

    private function getProductTypes(){
        $category_brand = array();

        $id_product = Tools::getValue('id_product');
        if(empty($id_product)){
            return null;
        }

        $product = new Product($id_product, false, $this->context->language->id);
        if(!empty($product->id)){
            $categories = Db::getInstance()->executeS('SELECT * FROM '._DB_PREFIX_.'category_product WHERE id_product = '.$product->id.' ORDER BY position');
            foreach($categories as $category){
                $brands = $this->getCategoryBreadcrumb($category['id_category']);
                if(!empty($brands)){
                    $category_brand[] = $brands;
                }
            }
        }

        return $category_brand;
    }

    public function getCategoryBreadcrumb($id_category){
        $context = Context::getContext();
        $path='';
        $pipe = '>';
        $full_path = '';

        $id_category = (int)$id_category;
        if ($id_category == 1) {
            return $path;
        }

        $interval = Category::getInterval($id_category);
        $id_root_category = $context->shop->getCategory();
        $interval_root = Category::getInterval($id_root_category);

        if ($interval) {
            $sql = 'SELECT c.id_category, cl.name, cl.link_rewrite
                    FROM '._DB_PREFIX_.'category c
                    LEFT JOIN '._DB_PREFIX_.'category_lang cl ON (cl.id_category = c.id_category'.Shop::addSqlRestrictionOnLang('cl').')
                    '.Shop::addSqlAssociation('category', 'c').'
                    WHERE c.nleft <= '.$interval['nleft'].'
                            AND c.nright >= '.$interval['nright'].'
                            AND c.nleft >= '.$interval_root['nleft'].'
                            AND c.nright <= '.$interval_root['nright'].'
                            AND cl.id_lang = '.(int)$context->language->id.'
                            AND c.active = 1
                            AND c.level_depth > '.(int)$interval_root['level_depth'].'
                    ORDER BY c.level_depth ASC';
            $categories = Db::getInstance()->executeS($sql);

            $n = 1;
            $n_categories = count($categories);
            foreach ($categories as $category) {
                $full_path .= htmlentities($category['name'], ENT_NOQUOTES, 'UTF-8').(($n++ != $n_categories || !empty($path)) ? $pipe : '');
            }

            return $full_path.$path;
        }

    }

    protected function getWidgetData($type = null, $id = null, $hook = null){

        $sql = 'SELECT * FROM `'._DB_PREFIX_.$this->name.'` WHERE `widget_enable` = 1 AND ';
        if(!empty($type)){
            $sql .= "`page_type` = '$type'";
        }else{
            $sql .= "`id_pswidget` = '$id'";
        }
        if($type == 'product'){
            $sql .= ' AND `product_hook` = "'.$hook.'"';
        }

        if (!$widget = Db::getInstance()->getRow($sql)){
            return;
        }

        if($widget['page_type'] == "category" && $type == "category" ){
            if($widget['show_in_category'] == 2){
                $id_category = Tools::getValue("id_category");
                $idcategories = explode(",", $widget['show_in_category_ids']);
                if(!in_array($id_category, $idcategories)){
                    return;
                }
            }
        }

        $widget['widget_version'] = 3;

        $widget['lang'] = $this->context->language->iso_code . '_' .$this->context->shop->id;

        if($widget['media_from_category'] == 1){
            if($widget['page_type'] == 'home'|| $widget['page_type'] == 'lookbook'){
                $idcategories = explode(",", $widget['media_from_category_ids']);
                $widget['productType'] = array();
                foreach($idcategories as $idcategory){
                    $widget['productType'][] = $this->getCategoryBreadcrumb($idcategory);
                }
            }
            if($widget['page_type'] == 'category'){
            	$widget['productType'][] = $this->getCategoryBreadcrumb(Tools::getValue("id_category"));
            }
            if($widget['page_type'] == 'product'){
                $widget['productType'] = $this->getProductTypes();
            }
        }

        if($type == 'product'){
            $id_product = Tools::getValue('id_product');
            $product = new Product($id_product, false, $this->context->language->id);
            $widget['id_product'] = $product->reference;
        }

        return $widget;
    }

    public function hookPsWidget($params){
        if(!$widget = $this->getWidgetData(null, $params['id_pswidget']))
            return;

        $this->smarty->assign('widget', $widget);
        $this->smarty->assign('json', $this->getJson($widget));

        $this->smarty->assign('container_id', 'ps_container'.$widget['id_pswidget'] );
        $this->smarty->assign('path_css', __PS_BASE_URI__.'modules/'.$this->name.'/css/'.$widget['widget_id'].'.css');

        return $this->display(__FILE__, 'views/templates/front/pswidget.tpl');
    }


    public function hookDisplayHome(){
        if(!$widget = $this->getWidgetData('home')){
            return;
        }

        $this->smarty->assign('widget', $widget);

        $this->smarty->assign('container_id', 'ps_container'.$widget['id_pswidget'] );
        $this->smarty->assign('path_css', __PS_BASE_URI__.'modules/'.$this->name.'/css/'.$widget['widget_id'].'.css');

        return $this->display(__FILE__, 'views/templates/front/pswidget.tpl');
    }


    public function displayContentProductHook($hook){
        if ($id_product = (int)Tools::getValue('id_product')){
            $product = new Product($id_product, true, $this->context->language->id, $this->context->shop->id);
        }
        if (!Validate::isLoadedObject($product)){
            return;
        }
        if(!$widget = $this->getWidgetData('product', null, $hook)){
            return;
        }
        if($widget['product_hook']!= $hook){
            return;
        }
        if(!$widget_shops = $this->getShopsWidget($widget['id_pswidget'])){
            return;
        }
        $product_id_type = Configuration::get('PSWIDGET_PRODUCT_ID_TYPE');

        foreach($widget_shops as $w_shop){
            if($w_shop['id_shop'] == $this->context->shop->id){
                $widget['product_id'] = ($product_id_type && $product_id_type != 'sku') ? array($product->$product_id_type) : array((string)$product->id);
                $this->context->smarty->assign(array(
                    'widget' => $widget,
                    'json' => $this->getJson($widget),
                    'container_id' => 'ps_container'.$widget['id_pswidget'] ,
                    'path_css' => __PS_BASE_URI__.'modules/'.$this->name.'/css/'.$widget['widget_id'].'.css'
                ));
                return $this->display(__FILE__, 'views/templates/front/pswidget.tpl');
            }
        }
    }

    protected function  getShopsWidget($id_widget){
        $res = Db::getInstance()->executeS('Select id_shop from '._DB_PREFIX_.$this->name.'_shop where id_pswidget='.$id_widget);
        return $res;
    }


    public function hookDisplayLeftColumnProduct(){
       return $this->displayContentProductHook('left');
    }
    public function hookDisplayRightColumnProduct(){
       return $this->displayContentProductHook('right');
    }
    public function hookDisplayProductTabContent(){
      return $this->displayContentProductHook('product_tab_content');
    }
    public function hookDisplayProductContent(){
       return $this->displayContentProductHook('product_content');
   }
    public function hookDisplayFooterProduct(){
        return $this->displayContentProductHook('footer');
    }

    public function hookDisplayFooter(){
        if($this->context->controller->php_self == "category"){

            if(!$widget = $this->getWidgetData('category')){
                return;
            }
            if($widget['show_in_category'] == 2){
                $idscategories = explode(",", $widget['show_in_category_ids']);
                $current = Tools::getValue("id_category");
                if(!in_array($current, $idscategories)){
                    return;
                }
            }
            if(empty($widget['identify_list_content'])){
                return;
            }

            $this->smarty->assign('widget', $widget);
            $this->smarty->assign('json', $this->getJson($widget));

            $this->smarty->assign('container_id', 'ps_container'.$widget['id_pswidget'] );
            $this->smarty->assign('path_css', __PS_BASE_URI__.'modules/'.$this->name.'/css/'.$widget['widget_id'].'.css');

            $this->smarty->assign('identify_content', $widget['identify_list_content'] );

            return $this->display(__FILE__, 'views/templates/front/pswidget.tpl');
        }
    }

    public function hookDisplayOrderConfirmation($params)
    {
        if (version_compare(_PS_VERSION_, '1.7', '>=') === true) {
            $order = $params['order'];
            $currency = new Currency($order->id_currency);
        } else {
            $order = $params['objOrder'];
            $currency = $params['currencyObj'];
        }

        $db_product_id_type = Configuration::get('PSWIDGET_PRODUCT_ID_TYPE');

        $product_value_type = (isset($db_product_id_type) && $db_product_id_type != 'sku') ? $db_product_id_type : 'id_product';

        $data_product = array();
        foreach ($order->getProducts() as $product) {
            $data_product[$product[$product_value_type]] = array(
                'count' => $product['product_quantity'],
                'price' => $product['product_price_wt'],
                'currency' => $currency->iso_code,
            );
        }

        $data = array(
            'event' => 'widget_converted',
            'products' => $data_product,
            'order_id' => $order->id,
            'visitor_id' => $_COOKIE['ps_analytics'],
            'album_id' => (int)Configuration::get('PSWIDGET_SHOP_' . $order->id_shop . '_CAMPAIGN'),
        );

        $this->sendConversionTracking($data);
    }

    public function sendConversionTracking($data){
        $jsondata = json_encode($data);
        $ch = curl_init('http://api.photoslurp.com/v1/widgets/record/?album_id='.$data['album_id']);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 5);
        curl_setopt($ch, CURLOPT_TIMEOUT, 5);
        curl_setopt ($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            'Content-Type: application/json', 'Connection: Keep-Alive'
        ));
        curl_setopt ($ch, CURLOPT_POSTFIELDS, $jsondata);
        curl_setopt($ch,CURLOPT_RETURNTRANSFER,true);

        $respuesta = curl_exec ($ch);
        $error = curl_error($ch);
        curl_close ($ch);
    }


    protected function renderWidgetsList(){
        $helper = new HelperList();

        $helper->title = $this->l('Widgets');
        $helper->table = $this->name;
        $helper->no_link = true;
        $helper->shopLinkType = '';
        $helper->identifier = 'id_pswidget';
        $helper->actions = array('edit', 'delete');

        $values = WidgetForms::getWidgetsListValues();
        $helper->listTotal = count($values);
        $helper->tpl_vars = array('show_filters' => false);

        $helper->toolbar_btn['new'] = array(
            'href' => $this->context->link->getAdminLink('AdminModules', false)
                .'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name
                .'&newpswidget=1&token='.Tools::getAdminTokenLite('AdminModules'),
            'desc' => $this->l('Add new widget')
        );

        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            .'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;

        return $helper->generateList($values, WidgetForms::getWidgetsList());
    }


    protected function postProcessNewWidget(){

        if ($this->isNewWidgetValid() == true)
        {

            $fields = Db::getInstance()->executeS('SHOW COLUMNS FROM '._DB_PREFIX_.$this->name);
            $data = array();
            foreach ($fields as $field){
                if ($field['Field'] == 'id_pswidget') continue;

                if(Tools::getValue($field['Field']) === false){
                    $value = null;
                }elseif($field['Field'] == 'show_in_category_ids'){
                    $value = implode(",", Tools::getValue('show_in_category_ids'));
                }elseif($field['Field'] == 'media_from_category_ids'){
                    $value = implode(",", Tools::getValue('media_from_category_ids'));
                }else{
                    $value = pSQL(Tools::getValue($field['Field']));
                }
                $data[$field['Field']] = $value;
            }
            if (Db::getInstance()->insert($this->name, $data, true) != false)
            {
                $id_widget = Db::getInstance()->Insert_ID();
                $this->writeCss($data);
                $this->associateShopWidget($id_widget);
                return $this->setSuccessMessage('The widget has been successfully added.');
            }
            return $this->setErrorMessage('An error happened: the widget could not be added.');
        }
        return false;
    }

    protected function associateShopWidget($id_widget){

        foreach(Shop::getShops() as $shop){
            if(isset($_POST['shop_'.$shop['id_shop']])){
                $shop_selected[$shop['id_shop']]=1;
            }else{
                $shop_selected[$shop['id_shop']]=0;
            }

            if($shop_selected[$shop['id_shop']]==1){

                $res = Db::getInstance()->executeS('Select * from '._DB_PREFIX_.$this->name.'_shop where id_pswidget='.$id_widget.' and id_shop='.$shop['id_shop']);

                $data = array(
                    'id_pswidget' => $id_widget,
                    'id_shop' => $shop['id_shop'],
                );

                if(empty($res)){
                    Db::getInstance()->insert($this->name.'_shop', $data);
                }
            }else{
                Db::getInstance()->delete($this->name.'_shop', '`id_shop` = '.$shop['id_shop'].' and `id_pswidget` = '.$id_widget);
            }
        }
    }

    protected function postProcessUpdateWidget(){

        if (Tools::isSubmit('id_pswidget') == false)
            return false;

            $fields = Db::getInstance()->executeS('SHOW COLUMNS FROM '._DB_PREFIX_.$this->name);
            $data = array();

            foreach ($fields as $field){
                if ($field['Field'] == 'id_pswidget') continue;

                if(Tools::getValue($field['Field']) === false){
                    $value = null;
                }elseif($field['Field'] == 'show_in_category_ids'){
                    $value = implode(",", Tools::getValue('show_in_category_ids'));
                }elseif($field['Field'] == 'media_from_category_ids'){
                    $value = implode(",", Tools::getValue('media_from_category_ids'));
                }else{
                    $value = pSQL(Tools::getValue($field['Field']));
                }
                $data[$field['Field']] = $value;
            }

        if (Db::getInstance()->update($this->name, $data, '`id_pswidget` = '.Tools::getValue('id_pswidget'),0,true) != false){
            $id_widget = Tools::getValue('id_pswidget');
            $this->writeCss($data);
            $this->associateShopWidget($id_widget);

            return $this->setSuccessMessage('The widget has been updated.');
        }
        return $this->setErrorMessage('The widget has not been updated');
    }

    protected function postProcessDeleteWidget($id_pswidget)
    {
        $id_pswidget = Tools::getValue('id_pswidget');

        Db::getInstance()->execute('DELETE FROM '._DB_PREFIX_.$this->name.' WHERE `id_pswidget` = \''.(int)$id_pswidget.'\'');
        Db::getInstance()->execute('DELETE FROM '._DB_PREFIX_.$this->name.'_lang WHERE `id_widget` = \''.(int)$id_pswidget.'\'');
                Db::getInstance()->delete($this->name.'_shop', '`id_pswidget` = '.$id_pswidget);

        return Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', false)
            .'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name
            .'&token='.Tools::getAdminTokenLite('AdminModules'));
    }

    protected function processExport()
    {
        $all_shops_selected = Tools::getValue('shop');
        $product_id_type = Tools::getValue('product_id_type');
        if(empty($all_shops_selected)){
            return $this->setErrorMessage('You must select one or more stores for the product feed file to be exported.');
        }
        Configuration::updateValue('PSWIDGET_EXPORTED_SHOPS', implode("," , $all_shops_selected));
        Configuration::updateValue('PSWIDGET_PRODUCT_ID_TYPE', $product_id_type);
        $this->export_class = new Export($all_shops_selected, $this->local_path.'csv/');
        $this->export_class->exportProductsShop();
        $all_links = $this->export_class->getChecked_files($this->export_class->getCsvFiles());
    }

     /**
     * Fetch the content of $template_name inside the folder current_module/mails/current_iso_lang/ if found, otherwise in mails/current_iso_lang
     *
     * @param string  $template_name template name with extension
     * @param int $mail_type     Mail::TYPE_HTML or Mail::TYPE_TXT
     * @param array   $var           list send to smarty
     *
     * @return string
     */
    protected function getEmailTemplateContent($template_name, $mail_type, $var){
        $email_configuration = Configuration::get('PS_MAIL_TYPE');
        if ($email_configuration != $mail_type && $email_configuration != Mail::TYPE_BOTH) {
            return '';
        }

        $module_template_path = _PS_MODULE_DIR_.$this->name.'/mails'.DIRECTORY_SEPARATOR.$this->context->language->iso_code.DIRECTORY_SEPARATOR.$template_name;
        $default_mail_template_path = _PS_MAIL_DIR_.$this->context->language->iso_code.DIRECTORY_SEPARATOR.$template_name;

        if (Tools::file_exists_cache($module_template_path)) {
            $default_mail_template_path = $module_template_path;
        }

        if (Tools::file_exists_cache($default_mail_template_path)) {
            $this->context->smarty->assign('list', $var);
            return $this->context->smarty->fetch($default_mail_template_path);
        }

        return '';
    }

    protected function isNewWidgetValid(){
        if ((Tools::getValue('widget_id')) && (Tools::getValue('user_name')) && (Tools::getValue('page_type'))){
            return true;
        }
        return $this->setErrorMessage('Fill in the required fields.');
    }

    protected function setErrorMessage($message)
    {
        $this->_errors[] = $this->l($message);
        return false;
    }

    protected function setSuccessMessage($message)
    {
        $this->_successes[] = $this->l($message);
        return true;
    }

    protected function isFieldSpecified($form_value){
        if (is_array($form_value))
            return (bool)array_filter($form_value,  function ($el) {return !is_null($el);});
        return !is_null($form_value);
    }

    protected function renderForm($form, $form_values, $action, $cancel = false, $back_url = false, $update = false){
        if($back_url==false){
            $back_url = $this->context->link->getAdminLink('AdminModules', false)
            .'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='
            .$this->name.'&token='.Tools::getAdminTokenLite('AdminModules');
        }

        $inputs = $form[0]['form']['input'];

        foreach ($inputs as $key => $input){
            if(((isset($input['required']) && $input['required'] === false) || !isset($input['required']))){
                $form[0]['form']['input'][$key]['disabled'] = true;
            }
        }

        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = $action;

        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            .'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;


        if ($update == true){
            $helper->currentIndex .= '&id_pswidget='.(int)Tools::getValue('id_pswidget').'&updatepswidget';
                }

        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $form_values,
            'id_language' => $this->context->language->id,
            'languages' => $this->context->controller->getLanguages(),
            'back_url' => $back_url,
            'show_cancel_button' => $cancel,
        );

        return $helper->generateForm($form);
    }

    protected function renderConfigForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitPswidgetModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            .'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm()));
    }


    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        $config_form_options = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Settings'),
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Add Clicknshopit analytics'),
                        'name' => 'PSWIDGET_ClICKNSHOPIT',
                        'is_bool' => true,
                        'desc' => $this->l('This option will add an analytics tracking code to all pages of your site,
                                            allowing you to track conversions from Clicknshop.it regardless
                                            of the types of pages you are linking your photos to.'),
                        'values' => array(
                            array(
                                'id' => 'clicknshopit_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'clicknshopit_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );
        $shops_config_fields = array_reverse($this->getShopsConfigFormFields());
        foreach($shops_config_fields as $shop_config){
            array_unshift($config_form_options['form']['input'],$shop_config);
        }
        return $config_form_options;
    }

    protected function getShopsConfigFormFields(){
        $shops_config_items=array();
        foreach(Shop::getShops() as $shop){
          array_push($shops_config_items, array(
            'type' => 'text',
            'label' => $shop['name'] . " " .$this->l('campaign ID'),
            'name' => "PSWIDGET_SHOP_" . $shop['id_shop'] . "_CAMPAIGN",
            'desc' => $this->l("Please enter the campaign ID for store \"") . $shop['name'] . "\". " . $this->l("This is used for our conversion tracking pixel."),
            'use_default_opt' => false
            ));
        }
        return $shops_config_items;
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        $config_form_values = array(
            'PSWIDGET_ClICKNSHOPIT' => Configuration::get('PSWIDGET_ClICKNSHOPIT')
        );
        $shop_campaigns = array_reverse($this->getShopsCampaigns());
        foreach($shop_campaigns as $shop_campaign){
            foreach($shop_campaign as $key => $value){
                $config_form_values[$key] = $value;
            }
        }
        return $config_form_values;
    }

    protected function getShopsCampaigns(){
        $shop_campaign = array();
        foreach(Shop::getShops() as $shop){
            $value = array("PSWIDGET_SHOP_" . $shop['id_shop'] . "_CAMPAIGN" => Configuration::get("PSWIDGET_SHOP_" . $shop['id_shop'] . "_CAMPAIGN"));
            array_unshift($shop_campaign, $value);
        }
        return $shop_campaign;
    }

    /**
     * Save form data.
     */
    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();
        foreach (array_keys($form_values) as $key)
            Configuration::updateValue($key, Tools::getValue($key));
    }

    protected function writeCss($data){
        if(isset($data['style_custom'])){
            $data['style_custom'] = str_replace('\r\n', PHP_EOL, $data['style_custom']);
            $pathname = _PS_MODULE_DIR_.$this->name.'/css';

            if(!is_dir($pathname)){
                mkdir($pathname);
            }
            $fp = fopen($pathname.'/'.$data['widget_id'].'.css', 'w+');

            fwrite($fp, $data['style_custom']);
            fclose($fp);
        }
    }
    /**
     * Add the CSS & JavaScript files you want to be loaded in the BO.
     */
}